import { useDebugValue, useEffect, useCallback, useRef, createContext, createElement, useContext } from "react";
import { EditorView } from "@codemirror/view";
import { unstable_batchedUpdates } from "react-dom";
import { useSyncExternalStore } from "use-sync-external-store/shim";
function runTask(callback) {
  let task = callback;
  queueMicrotask(() => task == null ? void 0 : task());
  return () => task = null;
}
function createAsyncScheduler() {
  let cancelTask;
  return {
    request: (callback) => {
      cancelTask = runTask(() => {
        cancelTask = null;
        callback();
      });
    },
    cancel: () => {
      if (cancelTask) {
        cancelTask();
        cancelTask = null;
      }
    }
  };
}
function isFunction(value) {
  return typeof value === "function";
}
function createCodeMirror(config) {
  let prevState;
  let currentView = null;
  function createConfig() {
    return (isFunction(config) ? config : () => config)(prevState);
  }
  function createView(container) {
    return new EditorView({
      ...createConfig(),
      parent: container
    });
  }
  const viewChangeCallbacks = /* @__PURE__ */ new Set();
  function subscribeViewChange(callback) {
    viewChangeCallbacks.add(callback);
    return () => viewChangeCallbacks.delete(callback);
  }
  function publishViewChange() {
    viewChangeCallbacks.forEach((callback) => callback());
  }
  function setView(view) {
    if (view === currentView) {
      return;
    }
    if (currentView) {
      prevState = currentView.state;
      currentView.destroy();
    }
    currentView = view;
    publishViewChange();
  }
  const getView = () => currentView;
  const getServerView = () => null;
  const useView = () => {
    const view = useSyncExternalStore(subscribeViewChange, getView, getServerView);
    useDebugValue(view);
    return view;
  };
  const useViewEffect = (effect, deps) => {
    const view = useView();
    useEffect(() => {
      if (view) {
        return effect(view);
      }
    }, [view, ...deps]);
  };
  const useViewDispatch = () => useCallback((...specs) => {
    const view = getView();
    if (!view) {
      throw new TypeError("Cannot dispatch transaction without a view");
    }
    view.dispatch(...specs);
  }, []);
  function createContainerRef() {
    let currentContainer = null;
    const scheduler = createAsyncScheduler();
    return Object.seal({
      get current() {
        return currentContainer;
      },
      set current(container) {
        if (container === currentContainer) {
          return;
        }
        currentContainer = container;
        scheduler.cancel();
        scheduler.request(
          () => unstable_batchedUpdates(() => {
            setView(null);
            setView(container && createView(container));
          })
        );
      }
    });
  }
  let currentContainerRef;
  function getContainerRef() {
    return currentContainerRef != null ? currentContainerRef : currentContainerRef = createContainerRef();
  }
  const useContainerRef = () => {
    const containerRef = getContainerRef();
    useDebugValue(containerRef);
    return containerRef;
  };
  return {
    getView,
    useView,
    useViewEffect,
    useViewDispatch,
    useContainerRef
  };
}
function toPascalCase(str) {
  return str.split(/[-_]/).map((word) => word[0].toUpperCase() + word.slice(1)).join("");
}
function useSingleton(instance) {
  const instanceRef = useRef(null);
  if (instanceRef.current === null) {
    instanceRef.current = isFunction(instance) ? instance() : instance;
  }
  return instanceRef.current;
}
function createCodeMirrorWithContext(displayName) {
  const InternalCodeMirrorContext = createContext(null);
  const CodeMirrorProvider = ({ config, children }) => {
    const contextValue = useSingleton(() => createCodeMirror(config));
    return /* @__PURE__ */ createElement(
      InternalCodeMirrorContext.Provider,
      { value: contextValue },
      children
    );
  };
  if (displayName) {
    displayName = toPascalCase(displayName);
    CodeMirrorProvider.displayName = `${displayName}.Provider`;
    InternalCodeMirrorContext.displayName = `Internal${displayName}`;
  }
  const useCodeMirrorContext = () => {
    const contextValue = useContext(InternalCodeMirrorContext);
    if (!contextValue) {
      throw new Error(
        "could not find CodeMirrorContext value; please ensure the component is wrapped in a <Provider>"
      );
    }
    return contextValue;
  };
  const useGetView = () => {
    const { getView: getContextView } = useCodeMirrorContext();
    return getContextView;
  };
  const useView = () => {
    const { useView: useContextView } = useCodeMirrorContext();
    return useContextView();
  };
  const useViewEffect = (effect, deps) => {
    const { useViewEffect: useContextViewEffect } = useCodeMirrorContext();
    return useContextViewEffect(effect, deps);
  };
  const useViewDispatch = () => {
    const { useViewDispatch: useContextViewDispatch } = useCodeMirrorContext();
    return useContextViewDispatch();
  };
  const useContainerRef = () => {
    const { useContainerRef: useContextContainerRef } = useCodeMirrorContext();
    return useContextContainerRef();
  };
  return {
    Provider: CodeMirrorProvider,
    useContext: useCodeMirrorContext,
    useGetView,
    useView,
    useViewEffect,
    useViewDispatch,
    useContainerRef
  };
}
export {
  createCodeMirror,
  createCodeMirrorWithContext
};
//# sourceMappingURL=index.js.map
